@@ -117,6 +117,15 @@ Value ListItem::meta_index(State &ls, const Value &key)
     return Value(ls);
 }
 
+bool ListItem::meta_contains(State &ls, const Value &key)
+{
+  try {
+    return get_child(key.to_string()).valid();
+  } catch (String &e) {
+    return false;
+  }
+}
+
 Iterator::ptr ListItem::new_iterator(State &ls)
 {
   return QTLUA_REFNEW(ListIterator, ls, ListItem::ptr(*this));
@@ -190,6 +199,8 @@ void ListItem::insert_name(Item *item)
       } while (_child_hash.contains(name));
     }
 
+  item->_name = name;
+
   _child_hash.insert(name, item);
 }
 
@@ -198,6 +209,11 @@ bool ListItem::accept_child(const Item::ptr &item) const
   return true;
 }
 
+int ListItem::get_column_count() const
+{
+  return 1;
+}
+
 ListItem::ListItem()
   : _id_counter(1)
 {
